﻿# pgnCb2Scid.ps1 reads a PGN in ChessBase format and displays it

$PgnInput = @'
[Event "?"]
[Site "?"]
[Date "2018.??.??"]
[Round "?"]
[White "*EXERCICES*"]
[Black "?"]
[Result "*"]
[PlyCount "0"]
[EventDate "2018.??.??"]
[EventType "swiss"]
[SourceDate "2018.12.23"]
[SourceVersionDate "2018.12.23"]

 *

'@

# Extract the game terminator at the end of moves
function Get-GameTerminator([string]$strMoves)
{
if ($strMoves.EndsWith(" *")){return "*"}
elseif ($strMoves.EndsWith(" 1/2-1/2")){return "1/2-1/2"}
elseif ($strMoves.EndsWith(" 1-0")){return "1-0"}
elseif ($strMoves.EndsWith(" 0-1")){return "0-1"}
else {return ""}
}

# Split moves according to one comment or variation by line
function Split-Comment([string]$strMoves)
{
    $strLines = ""
    $strMoves  -split "}" | ForEach-Object {
        $lineUntilEndOfCmt = $_
        if ($lineUntilEndOfCmt.Length -eq 0)
        {
            if ($strLines.Length -gt 0) # If there is at least one comment?
            {
                $strLines += "}" # Put a breakpoint using F9
            }
        }
        elseif ($strLines.Length -eq 0)
        {
            $strLines += $lineUntilEndOfCmt
        }
        else 
        {
            $strNextLine = $lineUntilEndOfCmt.Trim()
            if ($strNextLine.StartsWith(") ("))
            {
                $strLines += "})" + $cr + $strNextLine.Substring(2)

            }
            else
            {
                if ($strNextLine.Contains(") ("))
                {
                    $strNextLine = $strNextLine.Replace(") (", ")" + $cr + "(")
                }
                if ($strNextLine -eq ")") # Are the moves ending on a closing variation?
                {
                    $strLines += "})"
                }
                elseif ($strNextLine.StartsWith(") "))
                {
                    $strLines += "})" + $cr + $strNextLine.Substring(2)
                }
                else
                {
                    $strLines += "}" + $cr + $strNextLine
                }
            }
        }
    }
    return $strLines
}

# Display a warning and stop
function Warning([string]$strMsg)
{
    Write-output($strMsg)
    break
}

# Convert a CB one letter color to a Scid color
function GenColor([char]$abbrColor)
{
    if ($abbrColor -eq 'G'){return "green"}
    elseif ($abbrColor -eq 'Y'){return "yellow"}
    elseif ($abbrColor -eq 'R'){return "red"}
    else {Warning("2000: unknown color " + $abbrColor)}
}

# Generate in the PGN comment how to fill the given square with the color
function GenDrawFull([string]$strPrms)
{
    $strColor = GenColor($strPrms.Substring(0,1))
    return "[%draw full," + $strPrms.Substring(1,2) + "," + $strColor + "]"
}

# Generate in the PGN comment how to draw an arrow from a given square to another
function GenDrawArrow([string]$strPrms)
{
    $strColor = GenColor($strPrms.Substring(0,1))
    return "[%draw arrow," + $strPrms.Substring(1,2) + "," + $strPrms.Substring(3,2) +"," + $strColor + "]"
}

# Convert CB comment with drawing arrow or colored square to a Scid comment
function ConvCbToScid([string]$strPgnLine)
{
    $idxAction, $idxAfter = 0, 1
    ConvertFrom-String -InputObject $strPgnLine -Delimiter "\[%" -OutVariable PgnCmd -PropertyNames Before | Out-Null
    $strScidLine = $PgnCmd.Before
    $cmds = $PgnCmd | Get-Member -MemberType NoteProperty
    $cmds = $cmds[1..($cmds.count-1)] #Skip the first property "Before" at index 0 keeping all other properties
    $cmds | foreach-object {
        $PgnCmtRight = $PgnCmd.($_.Name) -split "\]", 2
        #Write-output("Cmd: " + $PgnCmtRight.Action)
        
        ConvertFrom-String -InputObject $PgnCmtRight[$idxAction] -TemplateContent $templateCmd -OutVariable PgnAction | Out-Null
        #Write-output("Cmd: " + $PgnAction.CmdName)
        $strPrms = $PgnAction.Prm.Replace(" ", "")
        #Write-output("Prm: " + $strPrms)
        if ($PgnAction.CmdName -eq "csl"){
            ConvertFrom-String -InputObject $strPrms -Delimiter "," -OutVariable PgnCmtDrawFull | Out-Null
            if ($strPrms.Contains(",")){
                $strPrms -split "," | ForEach-Object {$strScidLine += GenDrawFull($_)}
            }
            else
            {
                $strScidLine += GenDrawFull($strPrms)
            }            
        }
        elseif ($PgnAction.CmdName -eq "cal"){
            if ($strPrms.Contains(",")){
                $strPrms -split "," | ForEach-Object {$strScidLine += GenDrawArrow($_)}
            }
            else
            {
                $strScidLine += GenDrawArrow($strPrms)
            }
        }
        #break
        $strScidLine += $PgnCmtRight[$idxAfter]
    }
    return $strScidLine
}

# Main
$strTags, $strMoves = "", ""
$cr = "`r`n"
$isTag = 1
$PgnInput -split $cr | ForEach-Object{ # means For each line of text in the PGN where $_ represents the current line
    $strLine = $_
    if ($isTag -and ($strLine -match "^\[.*\]"))
    {
        $strTags += $strLine + $cr
    }
    elseif ($strLine.Length -eq 0)
    {
        if ($isTag)
        {
            $isTag = 0
            #$strTags = ""
        }
        else
        {
            $isTag = 1
        }
    }
    else
    {
        $strMoves += $strLine + " "
    }
}

# The last line contains the game terminator
$strMoves = $strMoves.TrimEnd()
$strTerminator = Get-GameTerminator($strMoves)
$lenTerminator = $strTerminator.Length
if ($lenTerminator -eq 0){Warning("1000: Missing game terminator")}
$strMoves = $strMoves.Substring(0, $strMoves.Length - $lenTerminator - 1)

$strLines = Split-Comment($strMoves)
$strMoves = ""

$templateCmd = @'
{CmdName*:csl} {Prm:Re5}
{CmdName*:csl} {Prm:Yh2,Yf4,Yc3,Gg4, Gd4,Gc5}
'@

Write-output($strTags)
$strLines -split $cr | ForEach-Object {
    if ($_.Contains("[%"))
    {
        #Write-output("Before: " + $_)
        $strConverted = ConvCbToScid($_)
        Write-output($strConverted)
    }
    else
    {
        Write-output($_)
    }
}
Write-output($strTerminator)
